@@ -16,6 +16,7 @@
  *   jl 05.10.97  Line speed changed to long in dial()
  *   jl 26.01.98  last login date & time added to dialing window
  *   jl 16.04.98  start searching for dialing tags from the highlighted entry
+ *   jl 12.06.98  save the old dialdir if it was an old version
  *
  */
 
@@ -109,6 +110,9 @@ struct dver {
   short res4;
 };
 
+/* Forward declaration */
+static void writedialdir();
+
 #define dialentno(di, no) ((struct dialent *)((char *)(di) + ((no) * sizeof(struct dialent))))  
 
 static struct dialent *dialents;
@@ -118,6 +122,8 @@ static int newtype;
 /* Access to ".dialdir" denied? */
 static int dendd = 0;
 static char *tagged;
+static char *dial_user;
+static char *dial_pass;
 
 /*
  * Functions to talk to the modem.
@@ -145,6 +151,26 @@ int how;
 			continue;
 		} else
   			c = (*s) & 31;
+	} else if (*s == '\\' && (*(s + 1))) {
+		s++;
+		switch (toupper (*s)) {
+			case '\\':
+			  c = *s;
+			  break;
+			case 'U':
+			  if (dial_user && *dial_user) 
+			    mputs (dial_user, how);
+			  s++;
+			  continue;
+			case 'P':
+			  if (dial_pass && *dial_pass)
+			    mputs (dial_pass, how);
+			  s++;
+			  continue;
+			default:
+			  s++;
+			  continue;
+		}
   	} else
   		c = *s;
 	if (how == 0 && c == '~')
@@ -164,7 +190,7 @@ void modeminit()
 
   if (P_MINIT[0] == '\0') return;
 
-  w = tell(_("Initializing Modem"));
+  w = mc_tell(_("Initializing Modem"));
   m_dtrtoggle(portfd, 1);         /* jl 23.06.97 */
   mputs(P_MINIT, 0);
   wclose(w, 1);
@@ -179,7 +205,7 @@ void modemreset()
 
   if (P_MRESET[0] == '\0') return;
 
-  w = tell(_("Resetting Modem"));
+  w = mc_tell(_("Resetting Modem"));
   mputs(P_MRESET, 0);
   sleep(1);
   wclose(w, 1);
@@ -192,8 +218,17 @@ void hangup()
 {
   WIN *w;
   int sec=1;
+  char buf[80];
+  extern time_t old_online;
 
-  w = tell(_("Hanging up"));
+  w = mc_tell(_("Hanging up"));
+
+  timer_update();
+  snprintf(buf, sizeof(buf),_("Hangup (%ld:%02ld:%02ld)"),
+	   online / 3600, (online / 60) % 60, online>0 ? online % 60 : 0);
+  do_log(buf);
+  online = -1;
+  old_online = -1;
 
   if (isdigit(P_MDROPDTR[0]))
     sscanf(P_MDROPDTR,"%2d",&sec);
@@ -204,10 +239,14 @@ void hangup()
   	mputs(P_MHANGUP, 0);
   	sleep(1);
   }
+#if _DCDFLOW
+  /* DCD has dropped, turn off hw flow control. */
+  m_sethwf(portfd, 0);
+#endif
   /* If we don't have DCD support fake DCD dropped */
   bogus_dcd = 0;
   wclose(w, 1);
-  do_log(_("Hangup"));   /* jl 22.06.97 */
+  time_status();
 }
 
 /*
@@ -218,7 +257,7 @@ void sendbreak()
 {
   WIN *w;
   
-  w = tell(_("Sending BREAK"));
+  w = mc_tell(_("Sending BREAK"));
   wcursor(w, CNONE);
 
   m_break(portfd);
@@ -264,7 +303,10 @@ void music()
   if (consolefd) close(consolefd);
 
   /* Wait for keypress and absorb it */
-  while((x & 2) == 0) x = check_io(-1, 0, 10000, NULL, NULL);
+  while((x & 2) == 0) {
+    x = check_io(-1, 0, 10000, NULL, NULL);
+    timer_update();
+  }
   (void) keyboard(KGETKEY, 0);
 }
 #endif
@@ -333,6 +375,12 @@ struct dialent **d2;
   char logline[128];
 
   timer_update(); /* Statusline may still show 'Online' / jl 16.08.97 */
+  /* don't do anything if already online! jl 07.07.98 */
+  if (P_HASDCD[0]=='Y' && online >= 0) {
+    werror(_("You are already online! Hang up first."));
+    return(retst);
+  }
+
   dialwin = wopen(18, 9, 62, 16, BSINGLE, stdattr, mfcolor, mbcolor, 0, 0, 1);
   wtitle(dialwin, TMID, _("Autodial"));
   wcursor(dialwin, CNONE);
@@ -426,7 +474,7 @@ MainLoop:
 		/* On space retry. */
 		if (f == ' ') {
 			mputs(P_MDIALCAN, 0);
-			dialfailed("Cancelled", 4);
+			dialfailed(_("Cancelled"), 4);
 			m_flush(portfd);
 			break;
 		}
@@ -450,6 +498,7 @@ MainLoop:
 		/* Only look when we got a whole line. */
 		if (modidx == 0 &&
 		    !strncmp(modbuf, P_MCONNECT, strlen(P_MCONNECT))) {
+			timer_update(); /* the login scipt may take long.. */
 			retst = 0;
 			/* Try to do auto-bauding */
 			if (sscanf(modbuf + strlen(P_MCONNECT), "%ld", &nb) == 1)
@@ -465,8 +514,8 @@ MainLoop:
 			do_log(logline);
 
 			ptime=localtime(&now);
-			sprintf(d->lastdate,"%02d%02d%02d",
-				ptime->tm_year,(ptime->tm_mon)+1,
+			sprintf(d->lastdate,"%4.4d%2.2d%2.2d",
+				(ptime->tm_year)+1900,(ptime->tm_mon)+1,
 				ptime->tm_mday);
 			sprintf(d->lasttime,"%02d:%02d",
 				ptime->tm_hour,ptime->tm_min);
@@ -480,7 +529,7 @@ MainLoop:
 			wlocate(dialwin, 1, 7);
 			if (d->script[0] == 0) {
 				wputs(dialwin,
-				     _("Connected. Press any key to continue"));
+				    _("Connected. Press any key to continue"));
 #if VC_MUSIC
 				if (P_SOUND[0] == 'Y')
 					music();
@@ -541,7 +590,7 @@ MainLoop:
 		wprintf(dialwin, "%-3d  ", dialtime);
 		if (dialtime <= 0) {
 			mputs(P_MDIALCAN, 0);
-			reason = "Timeout";
+			reason = _("Timeout");
 			retst = -1;
 			if (retries < maxretries) {
 				x = dialfailed(reason, rdelay);
@@ -679,10 +728,12 @@ int readdialdir()
   long size;
   FILE *fp;
   char dfile[256];
+  char copycmd[512];
   static int didread = 0;
   int f;
   struct dialent *d = NULL, *prev = (struct dialent *)0;
   struct dver dial_ver;
+  WIN *w;
 
   if (didread) return(0);
   didread = 1;
@@ -692,7 +743,7 @@ int readdialdir()
 
   /* Make the manual dial entry. */
   d_man = mkstdent();
-  strcpy(d_man->name, "Manually entered number");
+  strcpy(d_man->name, _("Manually entered number"));
 
   /* Construct path */
   snprintf(dfile, sizeof(dfile), "%s/.dialdir", homedir);
@@ -766,11 +817,7 @@ int readdialdir()
   if (size % dial_ver.size != 0) {
   	werror(_("Phonelist garbled (?)"));
   	fclose(fp);
-#if 0 /* This should be less facistic now. */
-  	unlink(dfile);
-#else
 	dendd = 1;
-#endif
   	dialents = mkstdent();
   	return(-1);
   }
@@ -823,6 +870,22 @@ int readdialdir()
   }
   d->next = (struct dialent *)0;
   fclose(fp);
+  if (dial_ver.size < sizeof(struct dialent)) {
+    if (snprintf(copycmd,sizeof(copycmd),
+	     "cp %s %s.%hd",dfile,dfile,dial_ver.size) > 0) {
+      do_log(copycmd);
+      if (system(copycmd) == 0) {
+	snprintf(copycmd,sizeof(copycmd),
+		 _("Old dialdir copied as %s.%hd"),dfile,dial_ver.size);
+	w=mc_tell(copycmd);
+	if (w) {
+	  sleep(2);
+	  wclose(w,1);
+	}
+	writedialdir();
+      }
+    }
+  }
   return(0);
 }
 
@@ -1072,7 +1135,7 @@ int position_dialing_directory, k;
 }
 
 
-static char *fmt = "\r %2d %c%-16.16s%-16.16s%6s %5s %4d %-15.15s\n";
+static char *fmt = "\r %2d %c%-16.16s%-16.16s%8.8s %5.5s %4d %-15.15s\n";
 
 /*
  * Print the dialing directory. Only draw from "cur" to bottom.
@@ -1117,7 +1180,7 @@ struct dialent *d;
 	mode_status();
   }
   newtype = d->term;
-  vt_set(-1, d->flags & FL_WRAP, NULL, -1, -1, d->flags & FL_ECHO, -1);
+  vt_set(-1, d->flags & FL_WRAP, NULL, -1, -1, d->flags & FL_ECHO, -1, -1);
 #ifdef _SELECT
   local_echo = d->flags & FL_ECHO;
 #endif
@@ -1150,6 +1213,8 @@ struct dialent *d;
   /* Remember _what_ we dialed.. */
   dial_name = d->name;
   dial_number = d->number;
+  dial_user = d->username;
+  dial_pass = d->password;
   return;
 }
 
@@ -1218,7 +1283,7 @@ void dialdir()
   wcursor(w, CNONE);
   wtitle(w, TMID, _("Dialing Directory"));
   wputs(w,
-	_("     Name            Number          Last on      Times Script\n"));
+      _("     Name            Number            Last on      Times Script\n"));
   wlocate(dsub, position_dialing_directory, 0);
   wputs(dsub, _(what));
 
@@ -1309,7 +1374,8 @@ again:
   			break;
   	}
   	/* Decide if we have to delete the cursor bar */
-  	if (cur != ocur || quit) wcurbar(w, ocur + 1 - top, XA_NORMAL | stdattr);
+  	if (cur != ocur || quit)
+	  wcurbar(w, ocur + 1 - top, XA_NORMAL | stdattr);
   		
   	if (cur < top) {
   		top--;
@@ -1474,7 +1540,7 @@ again:
   }
   /* Dial a number manually. */
   if (subm == 5) {
-	s = input("Enter number", manual);
+	s = input(_("Enter number"), manual);
 	if (s && *s) {
   		if (changed) writedialdir();
 		wclose(w, 1);
