@@ -15,6 +15,9 @@
  * fmg 8/20/97 History buffer searching added
  * jseymour@jimsun.LinxNet.com (Jim Seymour) 03/26/98 - Added support for
  *    multiple tty devices via new "get_port()" function.
+ * kubota@debian.or.jp 07/98  - Added option -C to start capturing from the
+ *				command line
+ * jl  09.07.98 added option -S to start a script at startup
  */
 
 #define EXTERN
@@ -94,7 +97,7 @@ int sig;
 
   wleave();
   signal(SIGTSTP, SIG_DFL);
-  printf(_("Suspended. Type \"fg\" to resume.\n"));
+  fputs(_("Suspended. Type \"fg\" to resume.\n"),stdout);
   kill(getpid(), SIGTSTP);
   signal(SIGTSTP, shjump);
   wreturn();
@@ -128,7 +131,7 @@ int dummy;
 	set_privs();
 	setgid((gid_t)real_gid);
   	setuid((uid_t)real_uid);
-	printf(_("Shelled out. Type \"exit\" to return.\n"));
+	fputs(_("Shelled out. Type \"exit\" to return.\n"), stdout);
   	execl(sh, sh, CNULL);
   	exit(1);
   }
@@ -145,6 +148,7 @@ WIN *w;
 int no;
 {
   int i;
+  static ELM outofrange[MAXCOLS] = {{0,0,0}};
 
   if (no < us->histlines) {
 	/* Get a line from the history buffer. */
@@ -156,7 +160,17 @@ int no;
 
   /* Get a line from the "us" window. */
   no -= us->histlines;
-  if (no >= w->ys) no = w->ys - 1;
+  if (no >= w->ys) {
+  	if (outofrange[0].value == 0) {
+  		for (i=0; i<MAXCOLS; i++) {
+  			outofrange[i].value = ' ';
+  			outofrange[i].color = us->color;
+  			outofrange[i].attr  = us->attr;
+  		}
+  		outofrange[0].value = '~';
+  	}
+  	return outofrange;
+  }
   return(w->map + (no * us->xs));
 }
 
@@ -192,7 +206,7 @@ int	case_matters;
 {
   int f;
   ELM *tmp_e;
-  char tmp_line[256];	/* should be enough */
+  char tmp_line[MAXCOLS];
 
   tmp_line[0]='\0';
   w->direct = 0;
@@ -244,18 +258,21 @@ char *str;
   wredraw(w_new, 1);
   wflush();
 
-  wlocate(w_new, strlen(hline)+6, y+1);
+  wlocate(w_new, strlen(hline)+6, 0); wputs(w_new, str);
+  wlocate(w_new, strlen(hline)+6, 0);
   (void) wgets(w_new, str, MAX_SEARCH, MAX_SEARCH);
+#if 0
   if (!str[0]) /* then unchanged... must have pressed ESC... get out */
   {
   	wflush();
   	wclose(w_new, 0);
   	return;
   }
+#endif
 
   wredraw(w_hist, 1);
+  wclose(w_new, 1);
   wflush();
-  wclose(w_new, 0);
 
   return;
 }
@@ -263,17 +280,20 @@ char *str;
 /*
  * fmg 8/20/97
  * Move scope to next hit of pattern in the buffer.
- * Returns line-number of next "hit_line" or -1 if none found (we beep elsewhere ;-)
+ * Returns line-number of next "hit_line" or -1 if none found
+ * (we beep elsewhere ;-)
  */
-int find_next(w, hit_line, look, case_matters)
+int find_next(w, w_hist, hit_line, look, case_matters)
 WIN	*w;
+WIN	*w_hist;
 int	hit_line;	/* 'current' Match line */
 char	*look;		/* pattern */
 int	case_matters;	/* guess... */
 {
   int next_line;
   ELM *tmp_e;
-  char tmp_line[256];	/* Should be more than enough... */
+  char tmp_line[MAXCOLS];
+  int all_lines;
 
   if (!look)
   	return(++hit_line); /* next line */
@@ -281,17 +301,18 @@ int	case_matters;	/* guess... */
   tmp_line[0]='\0';	/* Personal phobia, I need to do this.. */
 
   hit_line++;	/* we NEED this so we don't search only same line! */
+  all_lines = w->histlines + w_hist->ys;
 
-  if (hit_line >= us->histlines)	/* Make sure we've got a valid line! */
+  if (hit_line >= all_lines)	/* Make sure we've got a valid line! */
   {
 	werror(_("Search Wrapping Around to Start!"));
 	hit_line = 0;
   }
 
-  for (next_line = hit_line; next_line <= us->histlines; next_line++)
+  for (next_line = hit_line; next_line <= all_lines; next_line++)
   {
 	/* we do 'something' here... :-) */
-	tmp_e = getline(w, next_line);
+	tmp_e = getline(w_hist, next_line);
 
   	/*
 	 * First we "accumulate" the line into a variable.
@@ -305,7 +326,7 @@ int	case_matters;	/* guess... */
   			return(next_line);
   }
 
-  if (hit_line >= us->histlines)	/* Make sure we've got a valid line! */
+  if (hit_line >= all_lines)	/* Make sure we've got a valid line! */
   {
 	werror(_("Search Wrapping Around to Start!"));
 	hit_line = 0;
@@ -340,8 +361,8 @@ char *StrStr(str1, str2, case_matters)
 char	*str1, *str2;
 int	case_matters;
 {
-char	tmpstr1[256],
-	tmpstr2[256];
+char	tmpstr1[MAXCOLS],
+	tmpstr2[MAXCOLS];
 
 if (case_matters)
 	return (strstr(str1,str2));
@@ -352,6 +373,54 @@ else
 
 #endif /*_SEARCH_HISTORY*/
 
+static void drawcite(w, y, citey, start, end)
+WIN *w;
+int y;
+int citey;
+int start;
+int end;
+{
+  if (y+citey >= start && y+citey <= end) 
+	wdrawelm_inverse(w, y, getline(w, y+citey));
+  else
+	wdrawelm(w, y, getline(w, y+citey));
+}
+
+static void drawcite_whole(w, y, start, end)
+WIN *w;
+int y, start, end;
+{
+  int sy;
+  for (sy = 0; sy < w->ys; sy++) {
+	drawcite(w, sy, y, start, end);
+  }
+}
+
+static void do_cite(w, start, end)
+WIN *w;
+int start,end;
+{
+  char tmp_line[MAXCOLS];
+  ELM *tmp_e;
+  int x, y;
+
+  for (y=start; y<=end; y++) {
+	vt_send('>');
+	vt_send(' ');
+	tmp_e = getline(w, y);
+	wdrawelm_var(w, 0 /* dummy? */, tmp_e, tmp_line);
+	tmp_line[w->xs]=0;
+	for (x=w->xs-1; x>=0; x--){
+		if (tmp_line[x] <= ' ') tmp_line[x]=0;
+		else break;
+	}
+	for (x=0; tmp_line[x]; x++){
+		vt_send(tmp_line[x]);
+	}
+	vt_send(13);
+  }
+}
+
 /* Scroll back */
 static void scrollback()
 {
@@ -361,10 +430,16 @@ static void scrollback()
   ELM *tmp_e;
   int case_matters=0;	/* fmg: case-importance, needed for 'N' */
   static char look_for[MAX_SEARCH];	/* fmg: last used search pattern */
-  char tmp_line[256];
+  char tmp_line[MAXCOLS];
 #endif /*SEARCH_HISTORY*/
+  int citemode = 0;
+  int cite_ystart = 1000000,
+      cite_yend = -1,
+      cite_y;
+  int inverse;
+  int loop = 1;
 
-  char hline[128];
+  char hline0[128], hline1[128], *hline;
   static int hit=0;
 
   /* Find out how big a window we must open. */
@@ -390,13 +465,14 @@ static void scrollback()
    * added /=Srch, \=CaseLess, and N=Next and changed rest of line...
    * Hope you like it :-)
    */
-  strcpy(hline, _("HISTORY: U=Up1 D=Down1 F=PgDn B=PgUp s=Srch S=CaseLess N=NextHit ESC=Exit "));
+  strcpy(hline0, _("HISTORY: U=Up D=Down F=PgDn B=PgUp s=Srch S=CaseLess N=Next C=Cite ESC=Exit "));
 #else
-  strcpy(hline, "   SCROLL MODE    U=up D=down F=page-forward B=page-backward ESC=exit       ");
+  strcpy(hline0, "  SCROLL MODE    U=up D=down F=page-forward B=page-backward C=Cite ESC=exit ");
 #endif /*_HIST_SEARCH*/
 
   if (b_st->xs < 127)
-  	hline[b_st->xs] = 0;
+  	hline0[b_st->xs] = 0;
+  hline = hline0;
   wprintf(b_st, hline);
   wredraw(b_st, 1);
   wflush();
@@ -413,7 +489,8 @@ static void scrollback()
   
   drawhist(b_us, y, 0);
 
-  while((c = wxgetch()) != K_ESC) {
+  while(loop) {
+	c = wxgetch();
 	switch(c) {
 #if _SEARCH_HISTORY
 	  /*  
@@ -433,6 +510,7 @@ static void scrollback()
 		{ wbell(); werror(_("History buffer Disabled!")); break; }
   		if (!us->histline)
 		{ wbell(); werror(_("History buffer empty!")); break; }
+		if (citemode) break;
 		
 		/* we need this for the case-importance-toggle to work.. */
 		if (c == '/' || c == 's')
@@ -447,10 +525,11 @@ static void scrollback()
 		/* highlight any matches */
   		if (strlen(look_for)>1)
 		{
-  			hit = find_next(us, hit, look_for, case_matters);
+  			hit = find_next(us, b_us, y, look_for, case_matters);
 
 			if (hit == -1) { wbell(); wflush(); hit=0; break; }
   			drawhist_look(b_us, hit, 1, look_for, case_matters);
+  			y = hit;
 		}
   		else
 		{ wbell(); break; }
@@ -465,12 +544,14 @@ static void scrollback()
 	  case 'n':
 	  case 'N':
 		/* highlight NEXT match */
+		if (citemode) break;
   		if (strlen(look_for)>1)
 		{
-		  hit = find_next(us, hit, look_for, case_matters);
+		  hit = find_next(us, b_us, y, look_for, case_matters);
 
 		  if (hit == -1) { wbell(); wflush(); hit=0; break; }
 		  drawhist_look(b_us, hit, 1, look_for, case_matters);
+		  y = hit;
 		}
 		else	/* no search pattern... */
 		  {
@@ -484,8 +565,18 @@ static void scrollback()
 	  case 'u':
 	  case 'U':
 	  case K_UP:
+		if (citemode && cite_y) {
+			cite_y--;
+			if (cite_ystart != 1000000) {
+			  cite_yend = y + cite_y;
+			  drawcite(b_us, cite_y+1, y, cite_ystart, cite_yend);
+			  drawcite(b_us, cite_y, y, cite_ystart, cite_yend);
+			}
+			wlocate(b_us, 0, cite_y);
+			break;
+		}
 		if (y <= 0) break;
-		y--;
+		y--; if (cite_ystart != 1000000) cite_yend = y + cite_y;
 		wscroll(b_us, S_DOWN);
 
 #if _SEARCH_HISTORY
@@ -495,30 +586,50 @@ static void scrollback()
 		 * the lines that have the pattern we wanted... it's just nice.
 		 * This almost beggs for a function :-)
 		 */
-        	tmp_e = getline(b_us, y);
-		if (strlen(look_for) > 1)
-		{
-		  /* quick scan for pattern match */
-		  wdrawelm_var(b_us, 0, tmp_e, tmp_line);
-		  if (strlen(tmp_line)>1) {
-		    if (StrStr(tmp_line, look_for, case_matters))
-		      wdrawelm_inverse(b_us, 0, tmp_e);
-		    else
-		      wdrawelm(b_us, 0, tmp_e);
-		  }
+		if (citemode) {
+			inverse =
+			  (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
+		} else {
+        		tmp_e = getline(b_us, y);
+			if (strlen(look_for) > 1)
+			{
+			  /* quick scan for pattern match */
+			  wdrawelm_var(b_us, 0, tmp_e, tmp_line);
+			  inverse = (strlen(tmp_line)>1 &&
+			    StrStr(tmp_line, look_for, case_matters));
+			}
+        		else
+				inverse = 0;
 		}
-        	else
-                	wdrawelm(b_us, 0, tmp_e);
 #else
-                wdrawelm(b_us, 0, getline(b_us, y));
+		if (citemode) {
+			inverse =
+			  (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
+		} else {
+			inverse = 0;
+		}
 #endif /*_SEARCH_HISTORY*/
+		if (inverse) wdrawelm_inverse(b_us, 0, getline(b_us, y));
+		else wdrawelm(b_us, 0, getline(b_us, y));
+		if (citemode) wlocate(b_us, 0, cite_y);
 		wflush();
 		break;
 	  case 'd':
 	  case 'D':
 	  case K_DN:
+		if (citemode && cite_y < b_us->ys-1) {
+			cite_y++;
+			if (cite_ystart != 1000000) {
+			  cite_yend = y + cite_y;
+			  drawcite(b_us, cite_y-1, y, cite_ystart, cite_yend);
+			  drawcite(b_us, cite_y, y, cite_ystart, cite_yend);
+			}
+			wlocate(b_us, 0, cite_y);
+			break;
+		}
+			
 		if (y >= us->histlines) break;
-		y++;
+		y++; if (cite_ystart != 1000000) cite_yend = y + cite_y;
 		wscroll(b_us, S_UP);
 
 #if _SEARCH_HISTORY
@@ -528,23 +639,36 @@ static void scrollback()
 		 * the lines that have the pattern we wanted... it's just nice.
 		 * This almost beggs for a function :-)
 		 */
-        	tmp_e = getline(b_us, y + b_us->ys - 1);
-		if (strlen(look_for) > 1)
-		{
-			/* quick scan for pattern match */
-        		wdrawelm_var(b_us, 0, tmp_e, tmp_line);
-        		if (strlen(tmp_line)>1) {
-			  if (StrStr(tmp_line, look_for, case_matters))
-			    wdrawelm_inverse(b_us, b_us->ys - 1, tmp_e);
-			  else
-			    wdrawelm(b_us, b_us->ys - 1, tmp_e);
+		if (citemode) {
+			inverse =
+			  (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
+		} else {
+        		tmp_e = getline(b_us, y + b_us->ys - 1);
+			if (strlen(look_for) > 1)
+			{
+				/* quick scan for pattern match */
+        			wdrawelm_var(b_us, 0, tmp_e, tmp_line);
+				inverse = (strlen(tmp_line)>1 &&
+				  StrStr(tmp_line, look_for, case_matters));
 			}
+        		else
+				inverse = 0;
 		}
-        	else
-			wdrawelm(b_us, b_us->ys - 1, tmp_e);
 #else
-		wdrawelm(b_us, b_us->ys - 1, getline(b_us, y + b_us->ys - 1));
+		if (citemode) {
+			inverse =
+			  (y+cite_y >= cite_ystart && y+cite_y <= cite_yend);
+		} else {
+			inverse = 0;
+		}
 #endif /*_SEARCH_HISTORY*/
+		if (inverse)
+			wdrawelm_inverse(b_us, b_us->ys - 1, 
+			  getline(b_us, y + b_us->ys - 1));
+		else
+			wdrawelm(b_us, b_us->ys - 1, 
+			  getline(b_us, y + b_us->ys - 1));
+		if (citemode) wlocate(b_us, 0, cite_y);
 		wflush();
 		break;
 	  case 'b':
@@ -553,6 +677,7 @@ static void scrollback()
 		if (y <= 0) break;
 		y -= b_us->ys;
 		if (y < 0) y = 0;
+		if (cite_ystart != 1000000) cite_yend = y + cite_y;
 
 #if _SEARCH_HISTORY
 		/*
@@ -568,6 +693,7 @@ static void scrollback()
 #else
 			drawhist(b_us, y, 1);
 #endif /*_SEARCH_HISTORY*/
+		if (citemode) wlocate(b_us, 0, cite_y);
 		break;
 	  case 'f':
 	  case 'F':
@@ -575,8 +701,8 @@ static void scrollback()
 	  case K_PGDN:
 		if (y >= us->histlines) break;
 		y += b_us->ys;
-		if (y > us->histlines)
-		{ y=us->histlines; wbell(); break; }
+		if (y > us->histlines) y=us->histlines;
+		if (cite_ystart != 1000000) cite_yend = y + cite_y;
 
 #if _SEARCH_HISTORY
 		/*
@@ -592,10 +718,61 @@ static void scrollback()
 #else
 		drawhist(b_us, y, 1);
 #endif /*_SEARCH_HISTORY*/
+		if (citemode) wlocate(b_us, 0, cite_y);
+		break;
+	  case 'C': case 'c': /* start citation mode */
+		if (citemode ^= 1) {
+			cite_y = 0;
+			cite_ystart = 1000000;
+			cite_yend = -1;
+			strcpy(hline1, _("  CITATION: ENTER=select start line ESC=exit                               "));
+			if (b_st->xs < 127) hline1[b_st->xs]=0;
+			hline = hline1;
+		} else {
+			hline = hline0;
+		}
+		wlocate(b_st, 0, 0);
+		wprintf(b_st, hline);
+		wredraw(b_st, 1);
+		if (citemode) wlocate(b_us, 0, cite_y);
+		break;
+	  case 10: case 13:
+		if (!citemode) break;
+		if (cite_ystart == 1000000) {
+			cite_yend = cite_ystart = y + cite_y;
+			strcpy(hline1, _("  CITATION: ENTER=select end line ESC=exit                                 "));
+			if (b_st->xs < 127) hline1[b_st->xs]=0;
+		} else {
+			if (cite_ystart > cite_yend) break;
+			drawcite_whole(b_us, y, 1000000, -1);
+			loop = 0;
+			break;
+		}
+		wlocate(b_st, 0, 0);
+		wprintf(b_st, hline);
+		wredraw(b_st, 1);
+		wdrawelm_inverse(b_us, cite_y, getline(b_us, cite_ystart));
+		wlocate(b_us, 0, cite_y);
+		break;
+	  case K_ESC:
+		if (!citemode) {loop = 0; break;}
+		if (cite_ystart == 1000000) {
+			citemode = 0;
+			hline = hline0;
+		} else {
+			cite_ystart = 1000000;
+			strcpy(hline1, _("  CITATION: ENTER=select start line ESC=exit                               "));
+		}
+		drawcite_whole(b_us, y, cite_ystart, cite_yend);
+		wlocate(b_st, 0, 0);
+		wprintf(b_st, hline);
+		wredraw(b_st, 1);
+		if (citemode) wlocate(b_us, 0, cite_y);
 		break;
 	}
   }
   /* Cleanup. */
+  if (citemode) do_cite(b_us, cite_ystart, cite_yend);
   wclose(b_us, y == us->histlines ? 0 : 1);
   wclose(b_st, 1);
   wlocate(us, us->curx, us->cury);
@@ -640,7 +817,7 @@ char **s;
 }
 
 static char *bletch = 
-  N_("Usage: minicom [-somlz] [-c on] [-a on] [-t TERM] [-d entry] [-p ttyp] [configuration]\n");
+  N_("Usage: minicom [-somlz8] [-c on] [-a on] [-t TERM] [-S scriptname] [-d entry]\n\t[-p ttyp] [-C capturefile] [configuration]\n");
 
 static void usage(env_args, optind, mc)
 int env_args, optind;
@@ -648,8 +825,8 @@ char *mc;
 {
   if (env_args >= optind && mc)
   	fprintf(stderr, _("Wrong option in environment MINICOM=%s\n"), mc);
-  fprintf(stderr, _(bletch));
-  fprintf(stderr, _("Type \"minicom -h\" for help.\n"));
+  fputs(_(bletch), stderr);
+  fputs(_("Type \"minicom -h\" for help.\n"), stderr);
   exit(1);
 }
 
@@ -663,22 +840,25 @@ static void helpthem()
   printf(_(" (compiled %s)"), __DATE__);
 #endif /*__DATE_*/
   printf(_("(c) Miquel van Smoorenburg\n\n%s\n"), _(bletch));
-  printf(_("  -s             : enter setup mode (only as root)\n"));
-  printf(_("  -o             : do not initialize modem & lockfiles at startup\n"));
-  printf(_("  -m             : use meta or alt key for commands\n"));
-  printf(_("  -l             : literal ; assume screen uses the IBM-PC character set\n"));
-  printf(_("  -L             : Ditto, but assume screen uses ISO8859\n")); 
-  printf(_("  -z             : try to use terminal's status line\n"));
-  printf(_("  -c [on | off]  : ANSI style color usage on or off\n"));
-  printf(_("  -a [on | off]  : use reverse or highlight attributes on or off\n"));
-  printf(_("  -t term        : override TERM environment variable\n"));
-  printf(_("  -d entry       : dial `entry' from the dialing directory\n"));
-  printf(_("  -p ttyp..      : connect to pseudo terminal\n"));
-  printf(_("  configuration  : configuration file to use\n\n"));
-  printf(_("These options can also be specified in the MINICOM environment variable.\n"));
+  fputs(_("  -s             : enter setup mode (only as root)\n"),stdout);
+  fputs(_("  -o             : do not initialize modem & lockfiles at startup\n"), stdout);
+  fputs(_("  -m             : use meta or alt key for commands\n"), stdout);
+  fputs(_("  -l             : literal ; assume screen uses the IBM-PC character set\n"), stdout);
+  fputs(_("  -L             : Ditto, but assume screen uses ISO8859\n"), stdout); 
+  fputs(_("  -z             : try to use terminal's status line\n"), stdout);
+  fputs(_("  -8             : 8bit clean mode (for Japanese, etc)\n"), stdout);
+  fputs(_("  -c [on | off]  : ANSI style color usage on or off\n"), stdout);
+  fputs(_("  -a [on | off]  : use reverse or highlight attributes on or off\n"), stdout);
+  fputs(_("  -t term        : override TERM environment variable\n"), stdout);
+  fputs(_("  -S scriptname  : run script at startup\n"), stdout);
+  fputs(_("  -d entry       : dial `entry' from the dialing directory\n"), stdout);
+  fputs(_("  -p ttyp..      : connect to pseudo terminal\n"), stdout);
+  fputs(_("  -C filename    : start capturing with the file name\n"), stdout);
+  fputs(_("  configuration  : configuration file to use\n\n"), stdout);
+  fputs(_("These options can also be specified in the MINICOM environment variable.\n"), stdout);
   printf(_("This variable is currently %s%s.\n"), mc ? _("set to ") : _("unset"),
 	mc ? mc : "");
-  printf(_("The LIBDIR to find the configuration files and the\n"));
+  fputs(_("The LIBDIR to find the configuration files and the\n"), stdout);
   printf(_("access file minicom.users is compiled as %s.\n\n"), LIBDIR);
 
 #if 0  /* More than 24 lines.. */
@@ -753,6 +933,8 @@ char **argv;
   scr_passwd[0] = 0;
   dial_name = (char *)NULL;
   dial_number = (char *)NULL;
+  dial_user = (char *)NULL;
+  dial_pass = (char *)NULL;
   size_changed = 0;
   escape = 1;
   cmd_dial = NULL;
@@ -776,6 +958,14 @@ char **argv;
     if (strncmp (console_encoding, "ISO-8859", 8) == 0)
     screen_iso++; */
 
+/* kubota@debian.or.jp 12/07/1998 */
+  {
+	char *lang = getenv("LANG");
+	if (lang!=NULL && (!strncmp(lang,"ja",2) || !strncmp(lang,"ko",2))) {
+		args[argk++] = "-8";
+	}
+  }
+
 /* MARK updated 02/17/95 default history buffer size */
   num_hist_lines = 256;
 
@@ -790,7 +980,7 @@ char **argv;
  	bufp = buf;
  	buf[79] = 0;
  	while(isspace(*bufp)) bufp++;
- 	while(*bufp) {
+ 	while(*bufp && argk < 19) {
  		for(s = bufp; !isspace(*bufp) && *bufp; bufp++)
  			;
  		args[argk++] = s;
@@ -800,15 +990,16 @@ char **argv;
   env_args = argk;
 
   /* Add command - line options */
-  for(c = 1; c < argc; c++) args[argk++] = argv[c];
+  for(c = 1; c < argc && argk < 19; c++) args[argk++] = argv[c];
   args[argk] = CNULL;
 
   do {
 	/* Process options with getopt */
-	while((c = getopt(argk, args, "zhlLsomMbc:a:t:d:p:")) != EOF) switch(c) {
+	while((c = getopt(argk, args, "8zhlLsomMbc:a:t:d:p:C:S:")) != EOF)
+	  switch(c) {
   		case 's': /* setup */
   			if (real_uid != 0 && real_uid != eff_uid) {
-		fprintf(stderr, _("minicom: -s switch needs root privilige\n"));
+		fputs(_("minicom: -s switch needs root privilige\n"), stderr);
 				exit(1);
 			}
   			dosetup = 1;
@@ -822,7 +1013,8 @@ char **argv;
 				optarg += 5;
 			if (strncmp(optarg, "tty", 3) != 0 ||
 			    !strchr("pqrstuvwxyz", optarg[3])) {
-				fprintf(stderr, _("minicom: argument to -p must be a pty\n"));
+				fputs(
+				 _("minicom: argument to -p must be a pty\n"), stderr);
 				exit(1);
 			}
 			snprintf(pseudo, sizeof(pseudo), "/dev/%s", optarg);
@@ -886,6 +1078,22 @@ char **argv;
 		case 'z': /* Enable status line. */
 			use_status = 1;
 			break;
+		case 'C': /* Capturing */
+			capfp = sfopen(optarg, "a");
+			if (capfp == NULL) {
+				werror(_("Cannot open capture file"));
+				exit(1);
+			}
+			docap = 1;
+			vt_set(addlf, -1, capfp, docap, -1, -1, -1, -1);
+			break;
+		case 'S': /* start Script */
+			strncpy(scr_name, optarg, 33);
+			break;
+		case '8': /* 8bit clean mode */
+			vt_set(-1, -1, (FILE*)NULL, -1, -1, -1, -1, 1);
+			screen_ibmpc++; screen_iso++;
+			break;
   		default:
   			usage(env_args, optind, mc);
   			break;
@@ -914,7 +1122,7 @@ char **argv;
 
   /* Get password file information of this user. */
   if ((pwd = getpwuid(real_uid)) == (struct passwd *)0) {
-  	fprintf(stderr, _("You don't exist. Go away.\n"));
+  	fputs(_("You don't exist. Go away.\n"), stderr);
   	exit(1);
   }
 
@@ -954,7 +1162,8 @@ char **argv;
 			fprintf(stderr,
    _("Sorry %s. You are not allowed to use configuration %s.\n"),
 				pwd->pw_name, use_port);
-			fprintf(stderr, _("Ask your sysadm to add your name to %s\n"),
+			fprintf(stderr,
+				_("Ask your sysadm to add your name to %s\n"),
 				userfile);
 			exit(1);
 		}
@@ -1078,13 +1287,15 @@ char **argv;
 #if defined (__DATE__) && defined (__TIME__)
   wprintf(us, "%s %s, %s.\r\n",_("Compiled on"), __DATE__,__TIME__);
 #endif
-  wprintf(us, _("\nPress %sZ for help on special keys\n\n"), esc_key());
+  wprintf(us, _("\nPress %sZ for help on special keys%c\n\n"),esc_key(),'\r');
 
   /* Now that all initialization is done, drop our priviliges. */
   drop_privs();
 
   readdialdir();
 
+  if (scr_name[0]) runscript (0, scr_name, "", "");
+
   if (cmd_dial) dialone(cmd_dial);
 
   /* The main loop calls do_terminal and gets a function key back. */
@@ -1094,14 +1305,15 @@ dirty_goto:
 	switch(c + 32 *(c >= 'A' && c <= 'Z')) {
 		case 'a': /* Add line feed */
 			addlf = !addlf;
-			vt_set(addlf, -1, NULL, -1, -1, -1, -1);
-			s = addlf ? _("Add linefeed ON") : _("Add linefeed OFF");
+			vt_set(addlf, -1, NULL, -1, -1, -1, -1, -1);
+			s = addlf ?
+			  _("Add linefeed ON") : _("Add linefeed OFF");
 			werror(s);
 			break;
 #if _SELECT
 		case 'e': /* Local echo on/off. */
 			local_echo = !local_echo;
-			vt_set(-1, -1, NULL, -1, -1, local_echo, -1);
+			vt_set(-1, -1, NULL, -1, -1, local_echo, -1 ,-1);
 			s = local_echo ?
 			  _("Local echo ON") : _("Local echo OFF");
 			werror(s);
@@ -1123,6 +1335,11 @@ dirty_goto:
 			break;
 #endif
 		case 'm': /* Initialize modem */
+			if (P_HASDCD[0]=='Y' && online >= 0) {
+			  c = ask(_("You are online. Really initialize?"), c1);
+			  if (c != 0)
+			    break;
+			}
 			modeminit();
 			break;	
 		case 'q': /* Exit without resetting */
@@ -1189,7 +1406,7 @@ dirty_goto:
 				}
 				if (c == 1) docap = 0;
 			}
-			vt_set(addlf, -1, capfp, docap, -1, -1, -1);
+			vt_set(addlf, -1, capfp, docap, -1, -1, -1, -1);
 			break;
 		case 'p': /* Set parameters */
 			get_bbp(P_BAUDRATE, P_BITS, P_PARITY, 0);
@@ -1212,7 +1429,7 @@ dirty_goto:
 			break;
 		case 'w': /* Line wrap on-off */	
 			c = (!us->wrap);
-			vt_set(addlf, c, capfp, docap, -1, -1, -1);
+			vt_set(addlf, c, capfp, docap, -1, -1, -1, -1);
 			s = c ? _("Linewrap ON") : _("Linewrap OFF");
 			werror(s);
 			break;
