@@ -26,6 +26,12 @@
 #include "keyboard.h"
 #include "vt100.h"
 
+/*
+ * kubota@debian.or.jp 08/08/98
+ * COLS must be equal to or less than MAXCOLS.
+ */
+#define MAXCOLS 256
+
 #define XA_OK_EXIST	1
 #define XA_OK_NOTEXIST	2
 
@@ -76,6 +82,8 @@ EXTERN char *dial_tty;     /* tty to use. */
 EXTERN int portuid, portgid; /* Uid and Gid of /dev/ttyS... */
 EXTERN char *dial_name;	    /* System we're conneced to */
 EXTERN char *dial_number;   /* Number we've dialed. */
+EXTERN char *dial_user;     /* Our username there */
+EXTERN char *dial_pass;     /* Our password */
 
 /*
  * fmg 8/22/97
@@ -131,6 +139,10 @@ _PROTO(int domacsave, ( void ));	/* fmg - need it */
 _PROTO(int loadconv, ( char *));	/* jl */
 _PROTO(int saveconv, ( char *));	/* jl */
 
+/*	Prototypes from file: " common.c "    */
+_PROTO(char *pfix_home, ( char *s ));
+_PROTO(void do_log, ( char *line ));
+
 /*      Prototypes from file: " dial.c "      */
 
 _PROTO(void mputs, ( char *s , int how ));
@@ -182,7 +194,6 @@ _PROTO(void curs_status, ( void ));
 _PROTO(void show_status, ( void ));
 _PROTO(void scriptname, ( char *s ));
 _PROTO(int do_terminal, ( void ));
-_PROTO(void do_log, ( char *line ));
 
 /*      Prototypes from file: " minicom.c "      */
 
@@ -195,7 +206,7 @@ _PROTO(void port_init, ( ));
 #if _SEARCH_HISTORY
 _PROTO(void drawhist_look, (WIN *w, int y, int r, char *look, int case_matters));
 _PROTO(void searchhist, (WIN *w_orig, WIN *w_hist, int r, char *str));
-_PROTO(int find_next, (WIN *w, int hit_line, char *look, int case_matters));
+_PROTO(int find_next, (WIN *w, WIN *w_hist, int hit_line, char *look, int case_matters));
 _PROTO(const char *upcase, (char *dest, char *src));
 _PROTO(char *StrStr, (char *str1, char *str2, int case_matters));
 #endif /*SEARCH_HISTORY*/
@@ -257,7 +268,7 @@ _PROTO(void runscript, ( int ask, char *s , char *l , char *p ));
 /*      Prototypes from file: " windiv.c "      */
 
 /* Should use stdarg et al. */
-WIN *tell();
+WIN *mc_tell();
 void werror();
 _PROTO(int ask, ( char *what , char *s []));
 _PROTO(char *input, ( char *s , char *buf ));
