@@ -21,6 +21,7 @@
  * // acme 26.02.98 - i18n
  * // acme 18.03.98 - more i18n
  * // jl  05.04.98 - added the multifile parameter for transfer protocols
+ *    jl  06.07.98 - added option P_CONVCAP
  */
 #include "port.h"
 #include "minicom.h"
@@ -128,7 +129,7 @@ struct pars mpars[] = {
   { "",			PUBLIC,   "updir" },
   { "",			PUBLIC,   "downdir" },
   { "",			PUBLIC,   "scriptdir" },
-  { "Escape (Meta)",	PUBLIC,   "escape-key" },
+  { "^A",		PUBLIC,   "escape-key" },
   { "BS",		PUBLIC,   "backspace" },
   { N_("enabled"),	PUBLIC,   "statusline" },
   { N_("Yes"),		PUBLIC,   "hasdcd" },
@@ -159,6 +160,7 @@ struct pars mpars[] = {
 
   /* Character conversion table - jl / 04.09.97 */
   { "",                	PUBLIC,    "convf" },
+  { "Yes",		PUBLIC,	   "convcap" },
   /* Do you want to use the filename selection window? */
   { "Yes",		PUBLIC,	   "fselw" },
   /* Do you want to be prompted for the download directory? */
