@@ -4,19 +4,22 @@
  *		This program also looks like a basic interpreter :
  *		a bit messy. (But hey, I'm no compiler writer :-))
  *
- * Version:	1.21
+ * Version:	1.22
  *
  * Author:	Miquel van Smoorenburg, miquels@drinkel.ow.nl
  *
  * Bugs:	The "expect" routine is, unlike gosub, NOT reentrant !
  *
  *		This file is part of the minicom communications package,
- *		Copyright 1991-1995 Miquel van Smoorenburg.
+ *		Copyright 1991-1995 Miquel van Smoorenburg,
+ *		1997-1998 Jukka Lahtinen
  *
  *		This program is free software; you can redistribute it and/or
  *		modify it under the terms of the GNU General Public License
  *		as published by the Free Software Foundation; either version
  *		2 of the License, or (at your option) any later version.
+ *
+ * 10.07.98 jl  Added the log command
  */
 #include "port.h"
 #include "minicom.h"
@@ -27,7 +30,7 @@
 #define BREAK	2
 
 #ifndef lint
-char *Version = "@(#)runscript 1.21 05-Jul-1995 MvS";
+char *Version = "@(#)runscript 1.22 10-Jul-1998 JL";
 #endif
 
 struct line {
@@ -72,6 +75,7 @@ char *s_login = "name";		/* User's login name */
 char *s_pass = "password";	/* User's password */
 struct line *thisline;		/* Line to be executed */ 
 int laststatus = 0;		/* Status of last command */
+char homedir[256];		/* Home directory */
 
 static char inbuf[65];		/* Input buffer. */
 
@@ -880,6 +884,7 @@ struct kw {
   { "sleep",	dosleep },
   { "break",	dobreak },
   { "call",	docall },
+  { "log",	(int(*)())do_log },
   { (char *)0,	(int(*)())0 }
 };
  
@@ -946,6 +951,7 @@ int main(argc, argv)
 int argc;
 char **argv;
 {
+  char *s;
 #if 0 /* Shouldn't need this.. */
   signal(SIGHUP, SIG_IGN);
 #endif
@@ -953,9 +959,16 @@ char **argv;
   memset(inbuf, 0, sizeof(inbuf));
 
   if (argc < 2) {
-  	s_error("Usage: runscript <scriptfile>\r\n");
+  	s_error("Usage: runscript <scriptfile> [homedir]\r\n");
   	return(1);
   }
 
+  if (argc > 2)
+    strncpy(homedir, argv[2], sizeof(homedir));
+  else if ((s = getenv("HOME")) != CNULL)
+    strncpy(homedir, s, sizeof(homedir));
+  else
+    homedir[0]=0;
+
   return(execscript(argv[1]) != OK);
 }
