@@ -355,8 +355,13 @@ char attr, color;
   if (x < COLS && y < LINES) {
 #endif
      if (doit != 0) {
-  	_gotoxy(x, y);
-  	_setattr(attr, color);
+	static int x0=-1, y0=-1, c0=0;
+	static char attr0, color0;
+	if (x!=x0+1 || y!=y0 || attr!=attr0 || color!=color0 || !(c0&128)) {
+  		_gotoxy(x, y);
+  		_setattr(attr, color);
+	}
+	x0=x; y0=y; attr0=attr; color0=color; c0=c;
 	(void) outchar((screen_ibmpc || screen_iso || (attr & XA_ALTCHARSET)) ? c : wcharmap[(unsigned char)c]); 
 	/* a little test at 10.3.1998 / jl */
 	/* (void) outchar((unsigned char) c); */
@@ -605,6 +610,15 @@ int replace;
   wcursor(win, win->o_cursor);
   if (replace) {
 	for(y = win->y1; y <= win->y2; y++) {
+		/* temporal way to avoid 'half-character' problem */
+		/* in multibyte characters such as Japanese -- from here */
+		ELM *g;
+		g = gmap + (y * stdwin->xs);
+		for(x = 0 ; x < win->x1; x++) {
+			_write(g->value, 1, x, y, g->attr, g->color);
+			g++;
+		}
+		/* to here */
   		for(x = win->x1; x <= win->x2; x++) {
   			_write(e->value, 1, x, y, e->attr, e->color);
   			e++;
@@ -1055,6 +1069,7 @@ char *buf;
 	e++;
   }
 }
+#endif /*SEARCH_HISTORY*/
 
 /*
  * fmg 8/20/97
@@ -1087,7 +1102,6 @@ ELM *e;
   x = w->x2;
   _write(e->value, w->direct, x, y + w->y1, XA_NORMAL, e->color);
 }
-#endif /*SEARCH_HISTORY*/
 
 /*
  * Print a string in a window.
